@@ -2,7 +2,7 @@
 '''
 This module is a central location for all salt exceptions
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 # Import python libs
 import copy
@@ -11,7 +11,7 @@ import time
 
 # Import Salt libs
 import salt.defaults.exitcodes
-import salt.ext.six as six
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -39,8 +39,28 @@ class SaltException(Exception):
     Base exception class; all Salt-specific exceptions should subclass this
     '''
     def __init__(self, message=''):
-        super(SaltException, self).__init__(message)
-        self.strerror = message
+        # Avoid circular import
+        import salt.utils.stringutils
+        if not isinstance(message, six.string_types):
+            message = six.text_type(message)
+        if six.PY3 or isinstance(message, unicode):  # pylint: disable=incompatible-py3-code
+            super(SaltException, self).__init__(
+                salt.utils.stringutils.to_str(message)
+            )
+            self.message = self.strerror = message
+        elif isinstance(message, str):
+            super(SaltException, self).__init__(message)
+            self.message = self.strerror = \
+                salt.utils.stringutils.to_unicode(message)
+        else:
+            # Some non-string input was passed. Run the parent dunder init with
+            # a str version, and convert the passed value to unicode for the
+            # message/strerror attributes.
+            super(SaltException, self).__init__(str(message))  # future lint: blacklisted-function
+            self.message = self.strerror = unicode(message)  # pylint: disable=incompatible-py3-code
+
+    def __unicode__(self):
+        return self.strerror
 
     def pack(self):
         '''
@@ -48,7 +68,7 @@ class SaltException(Exception):
         transport via msgpack
         '''
         if six.PY3:
-            return {'message': str(self), 'args': self.args}
+            return {'message': six.text_type(self), 'args': self.args}
         return dict(message=self.__unicode__(), args=self.args)
 
 
@@ -100,7 +120,21 @@ class CommandExecutionError(SaltException):
     to show the user the output gracefully instead of dying
     '''
     def __init__(self, message='', info=None):
-        self.error = exc_str_prefix = message
+        # Avoid circular import
+        import salt.utils.stringutils
+        try:
+            exc_str_prefix = salt.utils.stringutils.to_unicode(message)
+        except TypeError:
+            # Exception class instance passed. The SaltException __init__ will
+            # gracefully handle non-string types passed to it, but since this
+            # class needs to do some extra stuff with the exception "message"
+            # before handing it off to the parent class' __init__, we'll need
+            # to extract the message from the exception instance here
+            try:
+                exc_str_prefix = six.text_type(message)
+            except UnicodeDecodeError:
+                exc_str_prefix = salt.utils.stringutils.to_unicode(str(message))  # future lint: disable=blacklisted-function
+        self.error = exc_str_prefix
         self.info = info
         if self.info:
             if exc_str_prefix:
@@ -133,6 +167,9 @@ class CommandExecutionError(SaltException):
         else:
             self.strerror_without_changes = exc_str = self.error
 
+        # We call the parent __init__ last instead of first because we need the
+        # logic above to derive the message string to use for the exception
+        # message.
         super(CommandExecutionError, self).__init__(exc_str)
 
 
@@ -164,8 +201,8 @@ class FileLockError(SaltException):
     '''
     Used when an error occurs obtaining a file lock
     '''
-    def __init__(self, msg, time_start=None, *args, **kwargs):
-        super(FileLockError, self).__init__(msg, *args, **kwargs)
+    def __init__(self, message, time_start=None, *args, **kwargs):
+        super(FileLockError, self).__init__(message, *args, **kwargs)
         if time_start is None:
             log.warning(
                 'time_start should be provided when raising a FileLockError. '
@@ -187,10 +224,9 @@ class GitLockError(SaltException):
     this exception class can be caught in a try/except without being caught as
     an OSError.
     '''
-    def __init__(self, errno, strerror, *args, **kwargs):
-        super(GitLockError, self).__init__(strerror, *args, **kwargs)
+    def __init__(self, errno, message, *args, **kwargs):
+        super(GitLockError, self).__init__(message, *args, **kwargs)
         self.errno = errno
-        self.strerror = strerror
 
 
 class GitRemoteError(SaltException):
@@ -226,26 +262,39 @@ class SaltRenderError(SaltException):
                  buf='',
                  marker='    <======================',
                  trace=None):
+        # Avoid circular import
+        import salt.utils.stringutils
         self.error = message
-        exc_str = copy.deepcopy(message)
+        try:
+            exc_str = salt.utils.stringutils.to_unicode(message)
+        except TypeError:
+            # Exception class instance passed. The SaltException __init__ will
+            # gracefully handle non-string types passed to it, but since this
+            # class needs to do some extra stuff with the exception "message"
+            # before handing it off to the parent class' __init__, we'll need
+            # to extract the message from the exception instance here
+            try:
+                exc_str = six.text_type(message)
+            except UnicodeDecodeError:
+                exc_str = salt.utils.stringutils.to_unicode(str(message))  # future lint: disable=blacklisted-function
         self.line_num = line_num
         self.buffer = buf
         self.context = ''
         if trace:
             exc_str += '\n{0}\n'.format(trace)
         if self.line_num and self.buffer:
-
-            import salt.utils
-            self.context = salt.utils.get_context(
+            # Avoid circular import
+            import salt.utils.templates
+            self.context = salt.utils.stringutils.get_context(
                 self.buffer,
                 self.line_num,
                 marker=marker
             )
             exc_str += '; line {0}\n\n{1}'.format(
                 self.line_num,
-                self.context
+                salt.utils.stringutils.to_unicode(self.context),
             )
-        SaltException.__init__(self, exc_str)
+        super(SaltRenderError, self).__init__(exc_str)
 
 
 class SaltClientTimeout(SaltException):
@@ -254,8 +303,8 @@ class SaltClientTimeout(SaltException):
 
     Takes the ``jid`` as a parameter
     '''
-    def __init__(self, msg, jid=None, *args, **kwargs):
-        super(SaltClientTimeout, self).__init__(msg, *args, **kwargs)
+    def __init__(self, message, jid=None, *args, **kwargs):
+        super(SaltClientTimeout, self).__init__(message, *args, **kwargs)
         self.jid = jid
 
 
@@ -265,6 +314,12 @@ class SaltCacheError(SaltException):
     '''
 
 
+class TimeoutError(SaltException):
+    '''
+    Thrown when an opration cannot be completet within a given time limit.
+    '''
+
+
 class SaltReqTimeoutError(SaltException):
     '''
     Thrown when a salt master request call fails to return within the timeout
@@ -341,7 +396,7 @@ class SaltCloudSystemExit(SaltCloudException):
     This exception is raised when the execution should be stopped.
     '''
     def __init__(self, message, exit_code=salt.defaults.exitcodes.EX_GENERIC):
-        SaltCloudException.__init__(self, message)
+        super(SaltCloudSystemExit, self).__init__(message)
         self.message = message
         self.exit_code = exit_code
 
@@ -383,6 +438,37 @@ class NotImplemented(SaltException):
     '''
 
 
+class TemplateError(SaltException):
+    '''
+    Used when a custom error is triggered in a template
+    '''
+
+
+class ArgumentValueError(CommandExecutionError):
+    '''
+    Used when an invalid argument was passed to a command execution
+    '''
+
+
+class CheckError(CommandExecutionError):
+    '''
+    Used when a check fails
+    '''
+
+
+# Validation related exceptions
+class InvalidConfigError(CommandExecutionError):
+    '''
+    Used when the config is invalid
+    '''
+
+
+class InvalidEntityError(CommandExecutionError):
+    '''
+    Used when an entity fails validation
+    '''
+
+
 # VMware related exceptions
 class VMwareSaltError(CommandExecutionError):
     '''
@@ -410,13 +496,61 @@ class VMwareObjectRetrievalError(VMwareSaltError):
     '''
 
 
+class VMwareObjectNotFoundError(VMwareSaltError):
+    '''
+    Used when a VMware object was not found
+    '''
+
+
+class VMwareObjectExistsError(VMwareSaltError):
+    '''
+    Used when a VMware object already exists
+    '''
+
+
+class VMwareMultipleObjectsError(VMwareObjectRetrievalError):
+    '''
+    Used when multiple objects were retrieved (and one was expected)
+    '''
+
+
+class VMwareNotFoundError(VMwareSaltError):
+    '''
+    Used when a VMware object was not found
+    '''
+
+
 class VMwareApiError(VMwareSaltError):
     '''
     Used when representing a generic VMware API error
     '''
 
 
+class VMwareFileNotFoundError(VMwareApiError):
+    '''
+    Used when representing a generic VMware error if a file not found
+    '''
+
+
 class VMwareSystemError(VMwareSaltError):
     '''
     Used when representing a generic VMware system error
     '''
+
+
+class VMwarePowerOnError(VMwareSaltError):
+    '''
+    Used when error occurred during power on
+    '''
+
+
+class VMwareVmRegisterError(VMwareSaltError):
+    '''
+    Used when a configuration parameter is incorrect
+    '''
+
+
+class VMwareVmCreationError(VMwareSaltError):
+    '''
+    Used when a configuration parameter is incorrect
+    '''
