@@ -15,7 +15,7 @@ In light of parted not directly supporting partition IDs, some of this module
 has been written to utilize sfdisk instead. For further information, please
 reference the man page for ``sfdisk(8)``.
 '''
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 
 # Import python libs
 import os
@@ -23,9 +23,11 @@ import stat
 import string
 import logging
 
-# Import salt libs
-import salt.utils
+# Import Salt libs
+import salt.utils.path
+import salt.utils.platform
 from salt.exceptions import CommandExecutionError
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -48,16 +50,16 @@ def __virtual__():
     Only work on POSIX-like systems, which have parted and lsblk installed.
     These are usually provided by the ``parted`` and ``util-linux`` packages.
     '''
-    if salt.utils.is_windows():
+    if salt.utils.platform.is_windows():
         return (False, 'The parted execution module failed to load '
                 'Windows systems are not supported.')
-    if not salt.utils.which('parted'):
+    if not salt.utils.path.which('parted'):
         return (False, 'The parted execution module failed to load '
                 'parted binary is not in the path.')
-    if not salt.utils.which('lsblk'):
+    if not salt.utils.path.which('lsblk'):
         return (False, 'The parted execution module failed to load '
                 'lsblk binary is not in the path.')
-    if not salt.utils.which('partprobe'):
+    if not salt.utils.path.which('partprobe'):
         return (False, 'The parted execution module failed to load '
                 'partprobe binary is not in the path.')
     return __virtualname__
@@ -92,7 +94,7 @@ def _validate_partition_boundary(boundary):
     '''
     try:
         for unit in VALID_UNITS:
-            if boundary.endswith(unit):
+            if six.text_type(boundary).endswith(unit):
                 return
         int(boundary)
     except Exception:
@@ -216,7 +218,7 @@ def align_check(device, part_type, partition):
             'Invalid partition passed to partition.align_check'
         )
 
-    cmd = 'parted -m -s {0} align-check {1} {2}'.format(
+    cmd = 'parted -m {0} align-check {1} {2}'.format(
         device, part_type, partition
     )
     out = __salt__['cmd.run'](cmd).splitlines()
@@ -377,16 +379,19 @@ def mkfs(device, fs_type):
     _validate_device(device)
 
     if fs_type not in set(['ext2', 'fat32', 'fat16', 'linux-swap', 'reiserfs',
-                          'hfs', 'hfs+', 'hfsx', 'NTFS', 'ufs']):
+                          'hfs', 'hfs+', 'hfsx', 'NTFS', 'ntfs', 'ufs']):
         raise CommandExecutionError('Invalid fs_type passed to partition.mkfs')
 
-    if fs_type is 'linux-swap':
+    if fs_type == 'NTFS':
+        fs_type = 'ntfs'
+
+    if fs_type == 'linux-swap':
         mkfs_cmd = 'mkswap'
     else:
         mkfs_cmd = 'mkfs.{0}'.format(fs_type)
 
-    if not salt.utils.which(mkfs_cmd):
-        return 'Error: {0} is unavailable.'
+    if not salt.utils.path.which(mkfs_cmd):
+        return 'Error: {0} is unavailable.'.format(mkfs_cmd)
     cmd = '{0} {1}'.format(mkfs_cmd, device)
     out = __salt__['cmd.run'](cmd).splitlines()
     return out
