@@ -111,6 +111,13 @@ def _p_to_cp(p):
     except portage.exception.InvalidAtom:
         pass
 
+    try:
+        ret = _porttree().dbapi.xmatch("match-all", p)
+        if ret:
+            return portage.cpv_getkey(ret[0])
+    except portage.exception.InvalidAtom:
+        pass
+
     return None
 
 
