@@ -3,26 +3,29 @@
 Manage basic template commands
 '''
 
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
-# Import python libs
+# Import Python libs
 import time
 import os
 import codecs
 import logging
 
-# Import salt libs
-import salt.utils
+# Import Salt libs
+import salt.utils.files
+import salt.utils.locales
 import salt.utils.stringio
-from salt.utils.odict import OrderedDict
+import salt.utils.versions
+
+# Import 3rd-party libs
 from salt.ext import six
 from salt.ext.six.moves import StringIO
 
 log = logging.getLogger(__name__)
 
 
-#FIXME: we should make the default encoding of a .sls file a configurable
-#       option in the config, and default it to 'utf-8'.
+# FIXME: we should make the default encoding of a .sls file a configurable
+#        option in the config, and default it to 'utf-8'.
 #
 SLS_ENCODING = 'utf-8'  # this one has no BOM.
 SLS_ENCODER = codecs.getencoder(SLS_ENCODING)
@@ -45,29 +48,24 @@ def compile_template(template,
     # if any error occurs, we return an empty dictionary
     ret = {}
 
-    log.debug('compile template: {0}'.format(template))
+    log.debug('compile template: %s', template)
 
     if 'env' in kwargs:
-        salt.utils.warn_until(
-            'Oxygen',
-            'Parameter \'env\' has been detected in the argument list.  This '
-            'parameter is no longer used and has been replaced by \'saltenv\' '
-            'as of Salt 2016.11.0.  This warning will be removed in Salt Oxygen.'
-            )
+        # "env" is not supported; Use "saltenv".
         kwargs.pop('env')
 
     if template != ':string:':
         # Template was specified incorrectly
         if not isinstance(template, six.string_types):
-            log.error('Template was specified incorrectly: {0}'.format(template))
+            log.error('Template was specified incorrectly: %s', template)
             return ret
         # Template does not exist
         if not os.path.isfile(template):
-            log.error('Template does not exist: {0}'.format(template))
+            log.error('Template does not exist: %s', template)
             return ret
         # Template is an empty file
-        if salt.utils.is_empty(template):
-            log.warning('Template is an empty file: {0}'.format(template))
+        if salt.utils.files.is_empty(template):
+            log.debug('Template is an empty file: %s', template)
             return ret
 
         with codecs.open(template, encoding=SLS_ENCODING) as ifile:
@@ -75,7 +73,7 @@ def compile_template(template,
             input_data = ifile.read()
             if not input_data.strip():
                 # Template is nothing but whitespace
-                log.error('Template is nothing but whitespace: {0}'.format(template))
+                log.error('Template is nothing but whitespace: %s', template)
                 return ret
 
     # Get the list of render funcs in the render pipe line.
@@ -85,14 +83,8 @@ def compile_template(template,
 
     input_data = StringIO(input_data)
     for render, argline in render_pipe:
-        # For GPG renderer, input_data can be an OrderedDict (from YAML) or dict (from py renderer).
-        # Repress the error.
-        if not isinstance(input_data, (dict, OrderedDict)):
-            try:
-                input_data.seek(0)
-            except Exception as exp:
-                log.error('error while compiling template \'{0}\': {1}'.format(template, exp))
-
+        if salt.utils.stringio.is_readable(input_data):
+            input_data.seek(0)      # pylint: disable=no-member
         render_kwargs = dict(renderers=renderers, tmplpath=template)
         render_kwargs.update(kwargs)
         if argline:
@@ -100,11 +92,10 @@ def compile_template(template,
         start = time.time()
         ret = render(input_data, saltenv, sls, **render_kwargs)
         log.profile(
-            'Time (in seconds) to render \'{0}\' using \'{1}\' renderer: {2}'.format(
-                template,
-                render.__module__.split('.')[-1],
-                time.time() - start
-            )
+            'Time (in seconds) to render \'%s\' using \'%s\' renderer: %s',
+            template,
+            render.__module__.split('.')[-1],
+            time.time() - start
         )
         if ret is None:
             # The file is empty or is being written elsewhere
@@ -112,17 +103,15 @@ def compile_template(template,
             ret = render(input_data, saltenv, sls, **render_kwargs)
         input_data = ret
         if log.isEnabledFor(logging.GARBAGE):  # pylint: disable=no-member
-            try:
-                log.debug('Rendered data from file: {0}:\n{1}'.format(
+            # If ret is not a StringIO (which means it was rendered using
+            # yaml, mako, or another engine which renders to a data
+            # structure) we don't want to log this.
+            if salt.utils.stringio.is_readable(ret):
+                log.debug(
+                    'Rendered data from file: %s:\n%s',
                     template,
-                    ret.read()))
-                ret.seek(0)
-            except Exception:
-                # ret is not a StringIO, which means it was rendered using
-                # yaml, mako, or another engine which renders to a data
-                # structure. We don't want to log this, so ignore this
-                # exception.
-                pass
+                    salt.utils.locales.sdecode(ret.read()))  # pylint: disable=no-member
+                ret.seek(0)  # pylint: disable=no-member
 
     # Preserve newlines from original template
     if windows_newline:
@@ -148,8 +137,8 @@ def compile_template_str(template, renderers, default, blacklist, whitelist):
     Take template as a string and return the high data structure
     derived from the template.
     '''
-    fn_ = salt.utils.mkstemp()
-    with salt.utils.fopen(fn_, 'wb') as ofile:
+    fn_ = salt.utils.files.mkstemp()
+    with salt.utils.files.fopen(fn_, 'wb') as ofile:
         ofile.write(SLS_ENCODER(template)[0])
     return compile_template(fn_, renderers, default, blacklist, whitelist)
 
@@ -172,26 +161,22 @@ def template_shebang(template, renderers, default, blacklist, whitelist, input_d
       #!mako|yaml_odict|stateconf
 
     '''
-    render_pipe = []
-
     line = ''
     # Open up the first line of the sls template
     if template == ':string:':
         line = input_data.split()[0]
     else:
-        with salt.utils.fopen(template, 'r') as ifile:
-            line = ifile.readline()
+        with salt.utils.files.fopen(template, 'r') as ifile:
+            line = salt.utils.stringutils.to_unicode(ifile.readline())
 
     # Check if it starts with a shebang and not a path
     if line.startswith('#!') and not line.startswith('#!/'):
-
         # pull out the shebang data
-        render_pipe = check_render_pipe_str(line.strip()[2:], renderers, blacklist, whitelist)
-
-    if not render_pipe:
-        render_pipe = check_render_pipe_str(default, renderers, blacklist, whitelist)
-
-    return render_pipe
+        # If the shebang does not contain recognized/not-blacklisted/whitelisted
+        # renderers, do not fall back to the default renderer
+        return check_render_pipe_str(line.strip()[2:], renderers, blacklist, whitelist)
+    else:
+        return check_render_pipe_str(default, renderers, blacklist, whitelist)
 
 
 # A dict of combined renderer (i.e., rend1_rend2_...) to
@@ -199,17 +184,15 @@ def template_shebang(template, renderers, default, blacklist, whitelist, input_d
 #
 OLD_STYLE_RENDERERS = {}
 
-for comb in '''
-        yaml_jinja
-        yaml_mako
-        yaml_wempy
-        json_jinja
-        json_mako
-        json_wempy
-        yamlex_jinja
-        yamlexyamlex_mako
-        yamlexyamlex_wempy
-        '''.strip().split():
+for comb in ('yaml_jinja',
+             'yaml_mako',
+             'yaml_wempy',
+             'json_jinja',
+             'json_mako',
+             'json_wempy',
+             'yamlex_jinja',
+             'yamlexyamlex_mako',
+             'yamlexyamlex_wempy'):
 
     fmt, tmpl = comb.split('_')
     OLD_STYLE_RENDERERS[comb] = '{0}|{1}'.format(tmpl, fmt)
@@ -221,6 +204,8 @@ def check_render_pipe_str(pipestr, renderers, blacklist, whitelist):
     If so, return the list of render functions in the pipe as
     (render_func, arg_str) tuples; otherwise return [].
     '''
+    if pipestr is None:
+        return []
     parts = [r.strip() for r in pipestr.split('|')]
     # Note: currently, | is not allowed anywhere in the shebang line except
     #       as pipes between renderers.
@@ -233,10 +218,13 @@ def check_render_pipe_str(pipestr, renderers, blacklist, whitelist):
             name, argline = (part + ' ').split(' ', 1)
             if whitelist and name not in whitelist or \
                     blacklist and name in blacklist:
-                log.warning('The renderer "{0}" is disallowed by cofiguration and will be skipped.'.format(name))
+                log.warning(
+                    'The renderer "%s" is disallowed by configuration and '
+                    'will be skipped.', name
+                )
                 continue
             results.append((renderers[name], argline.strip()))
         return results
     except KeyError:
-        log.error('The renderer "{0}" is not available'.format(pipestr))
+        log.error('The renderer "%s" is not available', pipestr)
         return []
